import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { Event } from "@mui/icons-material";
import { ClickAwayListener, useTheme, useMediaQuery } from "@mui/material";
import { LocalizationProvider, StaticDatePicker, } from "@mui/x-date-pickers-pro";
import { DateTime } from "luxon";
import { forwardRef, useEffect, useRef, useState, } from "react";
import { useTranslation } from "react-i18next";
import i18n from "../i18n";
import { NerdPickerWrapper } from "../nerdPickersWrapper";
import { CustomAdapterLuxon } from "./customAdapterLuxon";
import { FormControl, FormHelperText, FormLabel, Input, Box, IconButton, } from "@mui/joy";
const defaultlocaleStringFormat = {
    weekday: "long",
    month: "long",
    day: "2-digit",
};
/**
 *
 * @param {NerdDatePickerProps} props Props
 * @returns {ReactElement} NerdDateTimePicker
 */
export default forwardRef(function NerdDatePicker(props, parentRef) {
    const { setOpenCalendar, openCalendar, inputProps, helperText, label, datePickerProps, onChange, value, localeStringFormat = defaultlocaleStringFormat, formLabelProps, formControlProps, } = props;
    const [textFieldValue, setTextFieldValue] = useState(value !== null ? value.toLocaleString(localeStringFormat) : "");
    const [open, setOpen] = useState({
        anchorEl: null,
        triggeredBy: null,
    });
    const [tmpValue, setTmpValue] = useState(value);
    const theme = useTheme();
    const isDownMd = useMediaQuery(theme.breakpoints.down("md"));
    const { t } = useTranslation("components", { i18n: i18n });
    const inputRef = useRef(null);
    const iconButtonRef = useRef(null);
    const [disabledDate, setDisabledDate] = useState(false);
    const popperRef = useRef(null);
    const _helperText = ((value && !value?.isValid) || disabledDate) && !open.anchorEl
        ? t("nerdDatePicker.invalid_date")
        : helperText ?? "";
    const handleClick = (triggeredBy) => {
        if (!open.anchorEl) {
            setOpen({ anchorEl: iconButtonRef.current, triggeredBy });
            setOpenCalendar?.(true);
        }
        else
            handleClose();
    };
    const handleClose = () => {
        if (open.anchorEl !== null) {
            setOpen({ anchorEl: null, triggeredBy: null });
            setOpenCalendar?.(false);
        }
    };
    useEffect(() => {
        setTextFieldValue(value?.toLocaleString(localeStringFormat) || "");
        setTmpValue(value);
    }, [value]);
    useEffect(() => {
        if (openCalendar === true)
            setOpen({ anchorEl: iconButtonRef.current, triggeredBy: "input" });
        else
            handleClose();
    }, [openCalendar]);
    const onTextFieldValueChange = (newValue) => {
        setDisabledDate(false);
        setTextFieldValue(newValue);
        if (!newValue)
            onChange(null);
    };
    const onTextFieldBlur = (e) => {
        if (popperRef.current?.contains(e.relatedTarget) ||
            e.relatedTarget?.classList.contains("MuiPickersDay-root") ||
            e.relatedTarget?.classList.contains("DatePickerPopper")) {
            inputRef.current?.focus();
        }
    };
    const onDateChange = (newValue) => {
        if (newValue) {
            setTmpValue(newValue.isValid ? newValue : null);
            setTextFieldValue(newValue.toLocaleString(localeStringFormat));
            const newDate = value?.set({
                day: newValue.day,
                month: newValue.month,
                year: newValue.year,
            }) || newValue;
            if (newDate?.toMillis() !== value?.toMillis() && !isDownMd) {
                onChange(newDate || null);
            }
        }
    };
    const getActions = () => {
        const minDate = datePickerProps?.minDate;
        const today = DateTime.now();
        if (minDate) {
            if (typeof minDate !== "string") {
                return minDate > today ? [] : ["today"];
            }
            else {
                return DateTime.fromISO(minDate) > today ? [] : ["today"];
            }
        }
        else
            return ["today"];
    };
    return (_jsx(ClickAwayListener, { onClickAway: handleClose, children: _jsx("div", { children: _jsxs(LocalizationProvider, { dateAdapter: CustomAdapterLuxon, adapterLocale: "en-US", children: [_jsxs(FormControl, { ...formControlProps, children: [label !== "" ? (_jsx(FormLabel, { ...formLabelProps, children: label ?? t("nerdDatePicker.date") })) : null, _jsx(Input, { fullWidth: true, ...inputProps, slotProps: {
                                    ...inputProps?.slotProps,
                                    input: {
                                        ...inputProps?.slotProps?.input,
                                        ref: (ref) => {
                                            if (typeof parentRef === "function")
                                                parentRef(ref);
                                            else if (parentRef !== null)
                                                parentRef.current = ref;
                                            inputRef.current = ref;
                                        },
                                    },
                                }, autoComplete: "off", value: textFieldValue, endDecorator: _jsx(IconButton, { disabled: inputProps?.disabled ||
                                        formControlProps?.disabled ||
                                        inputProps?.readOnly, ref: iconButtonRef, onClick: (e) => {
                                        if (!inputProps?.disabled &&
                                            !formControlProps?.disabled &&
                                            !inputProps?.readOnly) {
                                            e.preventDefault();
                                            e.stopPropagation();
                                            handleClick("icon");
                                        }
                                    }, onKeyDown: (e) => {
                                        if (e.code == "Enter" &&
                                            !inputProps?.disabled &&
                                            !formControlProps?.disabled &&
                                            !inputProps?.readOnly) {
                                            e.stopPropagation();
                                            e.preventDefault();
                                            handleClick("icon");
                                        }
                                    }, children: _jsx(Event, {}) }), onClick: (e) => {
                                    if (!inputProps?.disabled &&
                                        !formControlProps?.disabled &&
                                        !inputProps?.readOnly)
                                        handleClick("input");
                                    inputProps?.onClick?.(e);
                                }, onChange: (e) => {
                                    onTextFieldValueChange(e.target.value);
                                    inputProps?.onChange?.(e);
                                }, onBlur: (e) => {
                                    onTextFieldBlur(e);
                                    inputProps?.onBlur?.(e);
                                }, onKeyDown: (e) => {
                                    if (e.code == "Tab" && !e.shiftKey) {
                                        e.stopPropagation();
                                    }
                                }, error: (value && !value?.isValid) ||
                                    disabledDate ||
                                    inputProps?.error
                                    ? true
                                    : false }), _helperText ? (_jsx(FormHelperText, { children: _helperText })) : null] }), _jsx(NerdPickerWrapper, { popperProps: {
                            ref: popperRef,
                            tabIndex: 0,
                            className: "DatePickerPopper",
                            open: open.anchorEl === null ? false : true,
                            anchorEl: open.anchorEl,
                            sx: {
                                zIndex: theme.zIndex.modal + 100,
                                boxShadow: theme.shadows[5],
                                borderRadius: "4px",
                                backgroundColor: "inherit",
                            },
                            onKeyDown: (e) => {
                                if (e.key === "Escape") {
                                    e.stopPropagation();
                                    handleClose();
                                    inputRef.current?.focus();
                                }
                                else if (e.key == "Tab") {
                                    e.stopPropagation();
                                }
                            },
                        }, dialogProps: {
                            open: open.anchorEl === null ? false : true,
                            onClose: handleClose,
                        }, dialogActionBarProps: {
                            onAccept() {
                                onChange(tmpValue);
                                handleClose();
                            },
                            onCancel() {
                                setTmpValue(value);
                                handleClose();
                            },
                            onClear() {
                                onChange(null);
                                handleClose();
                            },
                            onSetToday() {
                                onChange(DateTime.now());
                                handleClose();
                            },
                            actions: [...getActions(), "cancel", "accept"],
                        }, popperActionBarProps: {
                            onAccept() {
                                onChange(tmpValue);
                                handleClose();
                            },
                            onCancel() {
                                setTmpValue(value);
                                handleClose();
                            },
                            onClear() {
                                onChange(null);
                                handleClose();
                            },
                            onSetToday() {
                                onChange(DateTime.now());
                                handleClose();
                            },
                            actions: getActions(),
                        }, children: _jsx(Box, { sx: {
                                "& .MuiPickerStaticWrapper-content": {
                                    borderRadius: "4px",
                                },
                            }, children: _jsx(StaticDatePicker, { displayStaticWrapperAs: "desktop", slotProps: {
                                    actionBar: {
                                        actions: [],
                                    },
                                }, autoFocus: open.triggeredBy === "icon" ? true : false, value: tmpValue, ...datePickerProps, onChange: (value, keyboardInputValue) => {
                                    onDateChange(value);
                                    if (datePickerProps?.closeOnSelect !== false && !isDownMd) {
                                        handleClose();
                                        inputRef.current?.focus();
                                    }
                                    datePickerProps?.onChange?.(value, keyboardInputValue);
                                } }) }) })] }) }) }));
});
