import { jsx as _jsx } from "react/jsx-runtime";
import { Box, CircularProgress, Sheet } from "@mui/joy";
import { nanoid } from "@reduxjs/toolkit";
import { Suspense, lazy, useRef, useState, useMemo, useEffect, useCallback, } from "react";
import { createPortal } from "react-dom";
import { useSelector } from "react-redux";
import "react-resizable/css/styles.css";
import { nerdDraggableDialogIndexSelector, popNerdDraggableDialogOrder, pushNerdDraggableDialogOrder, } from "../redux/nerdDraggableDialog";
import { useDispatch } from "react-redux";
const Draggable = lazy(() => import("react-draggable"));
const Resizable = lazy(() => import("./reactResizable"));
/**
 *
 * @param {PropsWithChildren<NerdDraggableDialogProps>} props props
 * @returns {ReactElement} NerdDraggableDialog
 */
export function NerdDraggableDialog(props) {
    const { sheetProps, children, draggableProps, initialHeight, initialWidth, localStorageKeyPrefix, resizable, resizableProps, } = props;
    const rootRef = useRef(null);
    const uuid = useMemo(() => nanoid(), []);
    const index = useSelector((s) => nerdDraggableDialogIndexSelector(s, uuid));
    const dispatch = useDispatch();
    useEffect(() => {
        dispatch(pushNerdDraggableDialogOrder(uuid));
        return () => {
            dispatch(popNerdDraggableDialogOrder(uuid));
        };
    }, []);
    return createPortal(_jsx(Box, { sx: {
            zIndex: (theme) => index !== -1
                ? (theme.zIndex.modal + index + 1).toString()
                : theme.zIndex.modal + 1,
            position: "fixed",
            pointerEvents: "none",
        }, ref: rootRef, children: _jsx(Suspense, { fallback: _jsx(Box, { display: "flex", justifyContent: "center", width: "100%", sx: {
                    position: "fixed",
                    top: "50%",
                    left: "50%",
                    transform: "translate(-50%, -50%)",
                }, children: _jsx(Sheet, { sx: {
                        borderRadius: "md",
                        p: 3,
                        boxShadow: "xl",
                    }, ...sheetProps, children: _jsx(CircularProgress, {}) }) }), children: _jsx(Wrapper, { rootRef: rootRef, draggableProps: draggableProps, initialHeight: initialHeight, initialWidth: initialWidth, localStorageKeyPrefix: localStorageKeyPrefix, resizable: resizable, resizableProps: resizableProps, uuid: uuid, children: children }) }) }), document.body);
}
const Wrapper = ({ children, sheetProps, draggableProps, initialHeight: _initialHeight, initialWidth: _initialWidth, resizableProps, resizable, localStorageKeyPrefix, rootRef, uuid, }) => {
    const dispatch = useDispatch();
    const nodeRef = useRef(null);
    const [bounds, setBounds] = useState({
        left: 0,
        top: 0,
        bottom: 0,
        right: 0,
    });
    const _width = localStorageKeyPrefix &&
        localStorage.getItem(`${localStorageKeyPrefix}_width`)
        ? Number(localStorage.getItem(`${localStorageKeyPrefix}_width`))
        : undefined;
    const _height = localStorageKeyPrefix &&
        localStorage.getItem(`${localStorageKeyPrefix}_height`)
        ? Number(localStorage.getItem(`${localStorageKeyPrefix}_height`))
        : undefined;
    const initialWidth = _width || _initialWidth;
    const initialHeight = _height || _initialHeight;
    const [width, setWidth] = useState(initialWidth != undefined &&
        initialWidth <= window.document.documentElement.clientWidth
        ? initialWidth
        : window.document.documentElement.clientWidth - 100);
    const [height, setHeight] = useState(initialHeight != undefined &&
        initialHeight <= window.document.documentElement.clientHeight
        ? initialHeight
        : window.document.documentElement.clientHeight - 100);
    const getDefaultPosition = useCallback(() => {
        const { clientWidth, clientHeight } = window.document.documentElement;
        const targetRect = rootRef?.current?.getBoundingClientRect();
        if (targetRect && rootRef?.current) {
            return {
                x: clientWidth / 2 - rootRef.current.offsetWidth / 2,
                y: clientHeight / 2 - rootRef.current.offsetHeight / 2,
            };
        }
        else
            return undefined;
    }, [rootRef?.current]);
    const [position, setPosition] = useState();
    const onStart = (_event, uiData) => {
        const { clientWidth, clientHeight } = window.document.documentElement;
        const targetRect = nodeRef?.current?.getBoundingClientRect();
        if (targetRect)
            setBounds({
                left: -targetRect.left + uiData.x,
                right: clientWidth - (targetRect.right - uiData.x),
                top: -targetRect.top + uiData.y,
                bottom: clientHeight - (targetRect?.bottom - uiData.y),
            });
    };
    useEffect(() => {
        if (rootRef.current) {
            setPosition(getDefaultPosition());
        }
    }, []);
    return (_jsx(Draggable, { ...draggableProps, cancel: `.react-resizable-handle${draggableProps?.cancel ? `, ${draggableProps?.cancel}` : ""}`, nodeRef: nodeRef, bounds: bounds, onStart: onStart, position: position, onDrag: (_e, d) => setPosition({ x: d.x, y: d.y }), children: resizable ? (_jsx(Resizable, { ...resizableProps, width: width, height: height, onResize: (_e, data) => {
                setWidth(data.size.width);
                setHeight(data.size.height);
            }, onResizeStop: (_e, data) => {
                localStorage.setItem(`${localStorageKeyPrefix}_width`, data.size.width.toString());
                localStorage.setItem(`${localStorageKeyPrefix}_height`, data.size.height.toString());
            }, children: _jsx(Sheet, { ...sheetProps, sx: {
                    visibility: !position ? "hidden" : "visible",
                    borderRadius: "md",
                    p: 3,
                    boxShadow: "0 24px 38px 3px rgba(0,0,0,.14),0 9px 46px 8px rgba(0,0,0,.12),0 11px 15px -7px rgba(0,0,0,.2)",
                    ...sheetProps?.sx,
                    pointerEvents: "auto",
                    width,
                    height,
                }, slotProps: {
                    ...sheetProps?.slotProps,
                    root: {
                        ...sheetProps?.slotProps?.root,
                        ref: nodeRef,
                    },
                }, className: "nerdDraggableDialog", onFocus: () => {
                    dispatch(pushNerdDraggableDialogOrder(uuid));
                }, tabIndex: 0, children: children }) })) : (_jsx(Sheet, { ...sheetProps, sx: {
                visibility: !position ? "hidden" : "visible",
                borderRadius: "md",
                p: 3,
                boxShadow: "0px 0px 5px grey",
                ...sheetProps?.sx,
                pointerEvents: "auto",
            }, slotProps: {
                ...sheetProps?.slotProps,
                root: {
                    ...sheetProps?.slotProps?.root,
                    ref: nodeRef,
                },
            }, onFocus: () => {
                dispatch(pushNerdDraggableDialogOrder(uuid));
            }, tabIndex: 0, children: children })) }));
};
