import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import Slide from "@mui/material/Slide";
import Snackbar from "@mui/material/Snackbar";
import { Icon, Stack, Typography } from "@mui/material";
import Box from "@mui/material/Box";
import { useTranslation } from "react-i18next";
import nerdUII18n from "../../i18n";
import { Alert, Button, IconButton, LinearProgress } from "@mui/joy";
import { useSelector } from "react-redux";
import { nerdNotifierNotificationSelector } from "../../redux";
import { severityIcon } from "../const";
import CloseRoundedIcon from "@mui/icons-material/CloseRounded";
import { showNewMessage } from "@intercom/messenger-js-sdk";
import { useState, useRef, useEffect } from "react";
let direction = undefined;
/**
 *
 * @param {NerdNotificationProps & openHookAction} param0 components props
 * @returns {JSX.Element} NonIntrusive nerdNotifier
 */
export function NonIntrusive({ hideReportButton, handleClose, currentUser, }) {
    const notification = useSelector(nerdNotifierNotificationSelector);
    const { title, transition = "up", severity = "primary", subtitle, body, position = {
        vertical: "bottom",
        horizontal: "right",
    }, autoHideDuration = 5000, autohide, } = notification || {};
    direction = transition;
    const [progress, setProgress] = useState(0);
    const { t } = useTranslation("common", { i18n: nerdUII18n });
    const timer = useRef(0);
    useEffect(() => {
        timer.current = window.setInterval(() => {
            setProgress((prevProgress) => {
                return prevProgress >= 100
                    ? 0
                    : prevProgress + 100 / (autoHideDuration / 10);
            });
        }, 10);
        return () => {
            clearInterval(timer.current);
        };
    }, []);
    useEffect(() => {
        if (progress >= 100 && autohide) {
            clearInterval(timer.current);
            handleClose();
        }
    }, [progress, autohide]);
    return (_jsx(Box, { children: _jsx(Snackbar, { open: Boolean(notification), autoHideDuration: autohide ? autoHideDuration : null, TransitionComponent: Transition, anchorOrigin: position, children: _jsxs(Stack, { spacing: 1, children: [autohide && (_jsx(LinearProgress, { determinate: true, value: progress, color: severity })), _jsx(Alert, { variant: "solid", color: severity, sx: {
                            width: "100%",
                            alignItems: "center",
                        }, startDecorator: severityIcon[severity], endDecorator: _jsx(IconButton, { onClick: handleClose, sx: {
                                ml: 1,
                            }, variant: "solid", size: "sm", color: severity, children: _jsx(CloseRoundedIcon, {}) }), children: _jsxs("div", { children: [title ? (typeof title == "string" ? (_jsx(Typography, { color: "inherit", variant: "body1", children: title })) : (title)) : null, subtitle ? (typeof subtitle == "string" ? (_jsx(Typography, { color: "inherit", variant: "body2", children: subtitle })) : (subtitle)) : null, body ? (typeof body == "string" ? (_jsx(Typography, { color: "inherit", variant: "caption", children: body })) : (body)) : null] }) }), currentUser?.uuid &&
                        window._APP_CONFIG?.apiKeys?.intercom &&
                        (severity === "danger" || severity === "warning") &&
                        !hideReportButton ? (_jsx(Button, { size: "sm", onClick: () => {
                            showNewMessage(body
                                ? `Error: "${body}"\n(Please, do not delete this)\nYour message:\n`
                                : "");
                            handleClose();
                        }, startDecorator: _jsx(Icon, { fontSize: "small", className: "fa-regular fa-comment-exclamation" }), color: "neutral", children: t("components:nerdNotifier.report_issue") })) : null] }) }) }));
}
/**
 *
 * @param {TransitionProps} props Component props
 * @returns {JSX.Element} Transition Component
 */
function Transition(props) {
    return _jsx(Slide, { ...props, direction: direction });
}
