import { useCallback, useEffect, useLayoutEffect, useState, } from "react";
/**
 *
 * @param {MutableRefObject<GridApiPremium>} apiRef apiRef
 * @param {string} storageKey storageKey
 * @param {GridInitialState} defaultState defaultState
 * @param {DependencyList} deps deps
 * @returns {[number, () => void, () => void]} nerdUseSaveDataGridState
 */
export function useNerdSaveDataGridState(apiRef, storageKey, defaultState, deps) {
    const [key, setKey] = useState(0);
    const saveSnapshot = useCallback(() => {
        if (apiRef?.current?.exportState && localStorage) {
            const currentState = apiRef.current.exportState();
            localStorage.setItem(storageKey, JSON.stringify(currentState));
        }
    }, [apiRef]);
    useLayoutEffect(() => {
        // handle refresh and navigating away/refreshing
        window.addEventListener("beforeunload", saveSnapshot);
        return () => {
            // in case of an SPA remove the event-listener
            window.removeEventListener("beforeunload", saveSnapshot);
            saveSnapshot();
        };
    }, [saveSnapshot]);
    useEffect(() => {
        const stateFromLocalStorage = localStorage?.getItem(storageKey);
        apiRef?.current?.restoreState?.(stateFromLocalStorage
            ? JSON.parse(stateFromLocalStorage)
            : defaultState
                ? defaultState
                : {});
    }, [...(deps ? deps : [])]);
    const restoreState = useCallback(() => {
        if (apiRef?.current) {
            localStorage?.removeItem(storageKey);
            apiRef.current.restoreState?.(defaultState || {});
            setKey((key) => key + 1);
        }
    }, [apiRef]);
    return [key, restoreState, saveSnapshot];
}
